@@ -10,8 +10,9 @@ import aiopg
 import psycopg2
 import psycopg2.extras
 
-SYSTEM_ATTRIBUTES = ('TABLE_NAME', 'FRONTEND_EXCLUDE_FIELDS', 'SPECIAL_FIELDS', 'COMMON_FRONTEND_EXCLUDE_FIELDS',
-                     'frontend', 'COMMON_FRONTEND_INCLUDE_FIELDS', 'FRONTEND_INCLUDE_FIELDS')
+SYSTEM_ATTRIBUTES = ('TABLE_NAME', 'FRONTEND_EXCLUDE_FIELDS', 'SPECIAL_FIELDS', 'FRONTEND_INCLUDE_FIELDS', 'frontend', )
+FRONTEND_EXCLUDE_FIELDS = ('id', 'is_active', 'is_deleted', 'created', 'modified', )
+FRONTEND_INCLUDE_FIELDS = ()
 
 
 class DatabaseRecordNotFoundError(Exception):
@@ -28,6 +29,7 @@ class Relationship:
         self.field = relation_field
         self.key = key
         self.parameters = parameters
+        self.prefix = None
 
 
 class Relationships:
@@ -38,6 +40,7 @@ class Relationships:
         self.parameters = parameters
         self.sort_column = sort_column
         self.sort_order = sort_order
+        self.prefix = None
 
 
 class RelationCount:
@@ -46,6 +49,7 @@ class RelationCount:
         self.field = relation_field
         self.key = key
         self.parameters = parameters
+        self.prefix = None
 
 
 class QueryField:
@@ -54,9 +58,12 @@ class QueryField:
         self.parameters = parameters
 
 
-class Model:
-    COMMON_FRONTEND_EXCLUDE_FIELDS = ('id', 'is_active', 'is_deleted', 'created', 'modified', )
-    COMMON_FRONTEND_INCLUDE_FIELDS = ()
+class Model(object):
+    TABLE_NAME = None
+    FRONTEND_EXCLUDE_FIELDS = ()
+    FRONTEND_INCLUDE_FIELDS = ()
+    SPECIAL_FIELDS = ()
+
     id = 0
     uuid = None
     is_active = True
@@ -76,14 +83,22 @@ class Model:
             self.modified = datetime.datetime.utcnow()
 
     @classmethod
-    def table_short_name(cls):
-        name = f"{cls.TABLE_NAME}"
-        name = name.lower().replace('incase.', '')
+    def table_short_name(cls, short_prefix=None):
+        if not cls.TABLE_NAME:
+            raise ValueError(f"There is no TABLE_NAME attribute in class {cls.__name__}")
+
+        name = cls.TABLE_NAME.split('.')
+        name = name[-1].lower()
+        if short_prefix:
+            name = f"a_{short_prefix}_{name}"
         return name.strip()
 
     @classmethod
-    def table_full_name(cls):
-        return f"{cls.TABLE_NAME} {cls.table_short_name()}"
+    def table_full_name(cls, short_prefix=None):
+        if not cls.TABLE_NAME:
+            raise ValueError(f"There is no TABLE_NAME attribute in class {cls.__name__}")
+
+        return f"{cls.TABLE_NAME} {cls.table_short_name(short_prefix)}"
 
     @classmethod
     def attributes(cls, exclude_fields=()):
@@ -108,10 +123,11 @@ class Model:
     def sql_fields(cls, exclude_fields=(), db_exec=False):
         fields = []
         for attribute in cls.attributes(exclude_fields):
-            if hasattr(cls, 'SPECIAL_FIELDS') and attribute in cls.SPECIAL_FIELDS:
+            if attribute in cls.SPECIAL_FIELDS:
                 continue
             value = getattr(cls, attribute)
             if isinstance(value, QueryField) and not db_exec:
+                value.query = value.query.replace('#field=', f'{cls.table_short_name()}.')
                 fields.append(f'({value.query}) AS "{cls.table_short_name()}__{attribute}"')
                 continue
             elif isinstance(value, QueryField) and db_exec:
@@ -127,7 +143,7 @@ class Model:
         fields = []
         for attribute in cls.attributes(exclude_fields):
             value = getattr(cls, attribute)
-            if hasattr(cls, 'SPECIAL_FIELDS') and attribute in cls.SPECIAL_FIELDS:
+            if attribute in cls.SPECIAL_FIELDS:
                 continue
             if isinstance(value, QueryField):
                 continue
@@ -135,12 +151,12 @@ class Model:
         return ", ".join(fields)
 
     @staticmethod
-    def _process_parameter(table, field, value):
+    def _process_parameter(table, field, value, short_prefix=None):
         field_name = f"{field}_{''.join(random.choice(string.ascii_lowercase) for _ in range(6))}".replace('::', '_')
-        sql_name = f'{table.table_short_name()}.{field}'
+        sql_name = f'{table.table_short_name(short_prefix)}.{field}'
         try:
             if isinstance(getattr(table, field), QueryField):
-                sql_name = f'{table.table_short_name()}__{field}'
+                sql_name = f'{table.table_short_name(short_prefix)}__{field}'
         except AttributeError:
             if '::' not in field and '->' not in field and 'value between' in field.lower():
                 return [], []
@@ -217,7 +233,7 @@ class Model:
             fields = []
             values = []
             for item in value:
-                f, v = Model._process_parameter(table, field, item)
+                f, v = Model._process_parameter(table, field, item, short_prefix=short_prefix)
                 fields.append(f)
                 values.append(v)
             return [f"({' OR '.join(fields)})"], values
@@ -226,7 +242,7 @@ class Model:
         return [f"{sql_name} = %({field_name})s"], [{field_name: value}]
 
     @classmethod
-    def _process_parameters(cls, parameters, table_class=None):
+    def _process_parameters(cls, parameters, table_class=None, short_prefix=None):
         if not parameters:
             return "", {}
 
@@ -241,7 +257,7 @@ class Model:
         for parameter in parameters:
             if isinstance(parameter, dict):
                 for name, value in parameter.items():
-                    f, values = table_class._process_parameter(table_class, name, value)
+                    f, values = table_class._process_parameter(table_class, name, value, short_prefix=short_prefix)
                     fields.extend(f)
                     for v in values:
                         result_parameters.update(v)
@@ -249,7 +265,7 @@ class Model:
                 or_fields = []
                 for row in parameter:
                     for name, value in row.items():
-                        f, values = table_class._process_parameter(table_class, name, value)
+                        f, values = table_class._process_parameter(table_class, name, value, short_prefix=short_prefix)
                         or_fields.extend(f)
                         for v in values:
                             result_parameters.update(v)
@@ -262,7 +278,7 @@ class Model:
         return query, result_parameters
 
     @classmethod
-    def process_relationships(cls):
+    def process_relationships(cls, main_prefix=None, parent=None):
         fields = []
         parameters = {}
         for attribute in dir(cls):
@@ -271,26 +287,33 @@ class Model:
                     and not isinstance(value, RelationCount):
                 continue
             rel = value.relation
+            if cls.table_short_name(main_prefix) == rel.table_short_name():
+                value.prefix = ''.join(random.choice(string.ascii_lowercase) for _ in range(6))
+
             if isinstance(value, RelationCount):
-                query = f"SELECT COUNT(*) FROM {rel.table_full_name()} "
-                query += f"WHERE {rel.table_short_name()}.{value.field} = {cls.table_short_name()}.{value.key}"
-                where, params = cls._process_parameters(value.parameters, table_class=rel)
+                query = f"SELECT COUNT(*) FROM {rel.table_full_name(value.prefix)} "
+                query += f"WHERE {rel.table_short_name(value.prefix)}.{value.field} = " \
+                         f"{cls.table_short_name(main_prefix)}.{value.key}"
+                where, params = cls._process_parameters(value.parameters, table_class=rel, short_prefix=value.prefix)
                 if where:
                     query += f" AND {where}"
                     parameters.update(params)
-                fields.append(f"({query}) AS count__{cls.table_short_name()}__{attribute}")
+                fields.append(f"({query}) AS count__{cls.table_short_name(main_prefix)}__{attribute}")
                 continue
 
             rel_fields = []
             for rel_attr in rel.attributes():
-                if hasattr(rel, 'SPECIAL_FIELDS') and rel_attr in rel.SPECIAL_FIELDS:
+                if rel_attr in rel.SPECIAL_FIELDS:
                     continue
                 if isinstance(getattr(rel, rel_attr), QueryField):
+                    rel_val = getattr(rel, rel_attr)
+                    rel_val.query = rel_val.query.replace('#field=', f'{rel.table_short_name(value.prefix)}.')
                     rel_fields.append(f"'{rel.table_short_name()}__{rel_attr}', "
                                       f"({getattr(rel, rel_attr).query})")
                 else:
-                    rel_fields.append(f"'{rel.table_short_name()}__{rel_attr}', {rel.table_short_name()}.{rel_attr}")
-            rel_child_fields, params = rel.process_relationships()
+                    rel_fields.append(f"'{rel.table_short_name()}__{rel_attr}', "
+                                      f"{rel.table_short_name(value.prefix)}.{rel_attr}")
+            rel_child_fields, params = rel.process_relationships(value.prefix, parent=value)
             parameters.update(params)
             for rel_child_field in rel_child_fields:
                 json_select = rel_child_field[:rel_child_field.rindex(' AS ')]
@@ -301,19 +324,26 @@ class Model:
                 query = f"SELECT json_build_object({', '.join(rel_fields)}) "
             else:
                 query = f"SELECT json_agg(json_build_object({', '.join(rel_fields)})) "
-            query += f"FROM {rel.table_full_name()} "
-            query += f"WHERE {rel.table_short_name()}.{value.field} = {cls.table_short_name()}.{value.key}"
-            where, params = cls._process_parameters(value.parameters, table_class=rel)
+            query += f"FROM {rel.table_full_name(value.prefix)} "
+            if '->' in value.key:
+                combined_key = value.key.split('->')
+                query += f"WHERE {rel.table_short_name(value.prefix)}.{value.field} = " \
+                         f"(SELECT {combined_key[2]} FROM {combined_key[1]} " \
+                         f"WHERE id = {parent.relation.table_short_name(parent.prefix)}.{combined_key[0]})"
+            else:
+                query += f"WHERE {rel.table_short_name(value.prefix)}.{value.field} = " \
+                         f"{cls.table_short_name(main_prefix)}.{value.key}"
+            where, params = cls._process_parameters(value.parameters, table_class=rel, short_prefix=value.prefix)
             if where:
                 query += f" AND {where}"
                 parameters.update(params)
-            fields.append(f"({query}) AS json__{cls.table_short_name()}__{attribute}")
+            fields.append(f"({query}) AS json__{cls.table_short_name(main_prefix)}__{attribute}")
         return fields, parameters
 
     @staticmethod
     async def fetch(query, values=None, many=False, expected_result=True, postgres_uri=None):
         if not postgres_uri:
-            postgres_uri = os.getenv('DB_URL')
+            postgres_uri = os.getenv('POSTGRES_URI')
 
         try:
             pool = await aiopg.create_pool(postgres_uri, timeout=10)
@@ -342,13 +372,13 @@ class Model:
             return result
 
     @staticmethod
-    def _process_record(cls, record):
+    def _process_record(cls, record, prefix=None):
         if not record:
             return None
 
         columns = {}
         for attribute in cls.attributes():
-            if hasattr(cls, 'SPECIAL_FIELDS') and attribute in cls.SPECIAL_FIELDS:
+            if attribute in cls.SPECIAL_FIELDS:
                 continue
             record_key = f"{cls.table_short_name()}__{attribute}"
             columns[attribute] = record.get(record_key)
@@ -361,16 +391,16 @@ class Model:
             rel = value.relation
             if isinstance(value, RelationCount):
                 try:
-                    columns[attribute] = record.get(f"count__{cls.table_short_name()}__{attribute}", 0)
+                    columns[attribute] = record.get(f"count__{cls.table_short_name(prefix)}__{attribute}", 0)
                 except (ValueError, TypeError):
                     columns[attribute] = 0
                 continue
-            json_record_key = f"json__{cls.table_short_name()}__{attribute}"
+            json_record_key = f"json__{cls.table_short_name(prefix)}__{attribute}"
             json_record = record.get(json_record_key)
             if isinstance(value, Relationship):
-                columns[attribute] = Model._process_record(rel, json_record)
+                columns[attribute] = Model._process_record(rel, json_record, prefix=value.prefix)
             else:
-                columns[attribute] = Model._process_records(rel, json_record)
+                columns[attribute] = Model._process_records(rel, json_record, prefix=value.prefix)
                 if value.sort_column:
                     columns[attribute] = sorted(
                         columns[attribute],
@@ -381,10 +411,10 @@ class Model:
         return cls(**columns)
 
     @staticmethod
-    def _process_records(cls, records):
+    def _process_records(cls, records, prefix=None):
         objects = []
         for record in records if records else []:
-            objects.append(Model._process_record(cls, record))
+            objects.append(Model._process_record(cls, record, prefix=prefix))
         return objects
 
     @classmethod
@@ -432,7 +462,7 @@ class Model:
 
     @classmethod
     async def get_by_query(cls, query, parameters=None):
-        if not hasattr(cls, 'TABLE_NAME'):
+        if not cls.TABLE_NAME:
             raise ValueError(f"There is no TABLE_NAME attribute in class {cls.__name__}")
 
         record = await cls.fetch(query, parameters)
@@ -502,7 +532,7 @@ class Model:
         return count, cls._process_records(cls, records)
 
     async def insert(self):
-        if not hasattr(self, 'TABLE_NAME'):
+        if not self.TABLE_NAME:
             raise ValueError(f"There is no TABLE_NAME attribute in class {self.__name__}")
 
         fields = self.sql_fields(exclude_fields=('id',), db_exec=True)
@@ -511,7 +541,7 @@ class Model:
             field = field.replace('"', '').strip()
             values[field] = getattr(self, field)
 
-        query = f"INSERT INTO {getattr(self, 'TABLE_NAME')} " \
+        query = f"INSERT INTO {self.TABLE_NAME} " \
                 f"({', '.join(fields)}) " \
                 f"VALUES ({self.sql_format_fields(exclude_fields=('id',))}) RETURNING id"
 
@@ -519,7 +549,7 @@ class Model:
         self.id = record_id.get('id')
 
     async def update(self):
-        if not hasattr(self, 'TABLE_NAME'):
+        if self.TABLE_NAME:
             raise ValueError(f"There is no TABLE_NAME attribute in class {self.__name__}")
 
         self.modified = datetime.datetime.utcnow()
@@ -533,18 +563,18 @@ class Model:
 
         values['id'] = self.id
 
-        query = f"UPDATE {getattr(self, 'TABLE_NAME')} SET {', '.join(fields)} WHERE id = %(id)s"
+        query = f"UPDATE {self.TABLE_NAME} SET {', '.join(fields)} WHERE id = %(id)s"
         await self.fetch(query, values, expected_result=False)
 
     @classmethod
     async def delete(cls, pk=None, parameters=None):
         if not parameters:
-            query = f"DELETE FROM {getattr(cls, 'TABLE_NAME')} WHERE id = %(id)s"
+            query = f"DELETE FROM {cls.TABLE_NAME} WHERE id = %(id)s"
             await cls.fetch(query, {'id': pk}, expected_result=False)
             return
 
         where, parameters = cls._process_parameters(parameters)
-        query = f"DELETE FROM {getattr(cls, 'TABLE_NAME')} {cls.table_short_name()}"
+        query = f"DELETE FROM {cls.TABLE_NAME} {cls.table_short_name()}"
         if where:
             query += f" WHERE {where}"
         await cls.fetch(query, parameters, expected_result=False)
@@ -572,13 +602,8 @@ class Model:
                 result[attribute] = None
                 continue
 
-            include_fields = self.COMMON_FRONTEND_INCLUDE_FIELDS
-            if hasattr(self, 'FRONTEND_INCLUDE_FIELDS'):
-                include_fields += self.FRONTEND_INCLUDE_FIELDS
-            exclude_fields = self.COMMON_FRONTEND_EXCLUDE_FIELDS
-            if hasattr(self, 'FRONTEND_EXCLUDE_FIELDS'):
-                exclude_fields += self.FRONTEND_EXCLUDE_FIELDS
-
+            include_fields = FRONTEND_INCLUDE_FIELDS + self.FRONTEND_INCLUDE_FIELDS
+            exclude_fields = FRONTEND_EXCLUDE_FIELDS + self.FRONTEND_EXCLUDE_FIELDS
             if attribute not in include_fields and attribute in exclude_fields:
                 continue
 
